%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% This script is used to obtain the results in the paper
%%%% "Forest Sparsity for Multi-task Compressive Sensing"
%%%%% Jan. 21, 2013, Created by Chen Chen and Junzhou Huang in University
%%%%% of Texas at Arlington
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

close all; clear all; 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for iter=1:3,
load T2MRI;
addpath('Rice_Wavelet_Toolbox_2.4');
rand('state',0); randn('state',0);

X=T2MRI(:,:, 90+iter, 2:4);
X=squeeze(X); X=double(X);
T=size(X, 3);
m=256; n=m;
for t=1:T,
    temp=imresize(X(:,:,t), [m n]);
    temp=im256(temp);
    F(:,:,t)=temp;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% Preparing the data and operators
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[m n, T] = size(F); N = m*n; 
d = 4;%dd(iter);   %%%% choose 4, 4.4, 5, 6.3
% tree structure
DecLevel=4;
[IdxParent, IdxChildren, Ms] = WaveTreeStructure2D(m,DecLevel);
[G,Gmat,GmatX,groups]=GroupMatrics(IdxParent);
input.G=G;input.Gt=G';input.GtG=G'*G;
input.groups=groups;input.Gmat=Gmat;

wav = daubcqf(2);%W = @(x) midwt(x,wav,DecLevel);WT = @(x) mdwt(x,wav,DecLevel);
WT = @(x) reshape(mdwt(reshape(x,m,n),wav,DecLevel),N,1);
W = @(x) reshape(midwt(reshape(x,m,n),wav,DecLevel),N,1);
Phi= A_operator(@(x) WT(x), @(x) W(x));    % notice Phi=WT
 
for t=1:T,
     f2=F(:,:, t); f(:,t)=f2(:); xs(:,t)=WT(f(:,t));
     [OMEGA{t}] = RandMask_rect(double(m/d),double(n/d),m,n);
     k = length(OMEGA{t});
     [mask{t}] = RandMask_InverseTransfer(OMEGA{t},m,n); 
     
     R{t} = @(x) A_fhp_rect(x, OMEGA{t}, m, n);
     RT{t} = @(x) At_fhp_rect(x, OMEGA{t}, m, n);
     sigma = 0.01; noise = sigma*randn(2*k+1,1); 
     RR{t} = A_operator(@(x) R{t}(x), @(x) RT{t}(x));
     b{t} = RR{t}*f(:,t) + noise;
     
     
     data = zeros(m,n);data(1,1) = b{t}(1); KK = length(b{t});
     data(OMEGA{t}) = b{t}(2:(KK+1)/2) + i*b{t}((KK+3)/2:KK);
     im_poc{t} = real(ifft2(data))*sqrt(m*n);
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
alpha = 1e-10; beta = 3.5e-2;
input.f=f; input.xs=xs; input.n1=m;input.n2=n;
input.alpha=alpha;input.beta=beta; input.lamda=beta*0.5;
input.Phi=Phi;input.L=1;
input.maxitr=1000;input.maxiitr= 1000;
input.stopCriterion=2; 
input.tolA=1e-8; input.no=400; %%%%  Change no for different iteration numbers
input.l=-inf; input.u=inf; 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% FCSA  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf('calling the function  FISTA.....\n');

% figure(1);
for t=1:T,
    out = FISTA_MT(b{t}, RR{t}, f(:,t), input);
    img(:,t) = out.y; x1(:,t)=Phi*img(:, t); 
    SNR1(:,t)=out.snr'; Time1(:,t)=out.xtime;
%     fprintf(1,'Iter_time = %.2fsec, Iteration Num = %d, snr = %f, samp.ratio = %f \n',out.xtime(end), input.no, out.snr(end), k/(m*n));
end
im1=reshape(img, [m, n, T]);
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% FCSA_MT
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf('calling the function  FISTA_Joint.....\n');
input1=input; 
input1.alpha=alpha*sqrt(T);input1.beta=beta*sqrt(T);
out = FISTA_MT(b, RR, f, input1);
out2=out;img = out.y; 
for t=1:T, 
    x2(:,t)=Phi*img(:,t); 
end
im2=reshape(img, [m, n, T]);
SNR2=out.snr'; Time2=out.xtime;
fprintf(1,'Iter_time = %.2fsec, Iteration Num = %d, snr = %f, samp.ratio = %f \n',out.xtime(end), input.no, out.snr(end), k/(m*n));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% Tree
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf('calling the function  FISTA_Tree.....\n');
% figure(1);
for t=1:T,
    out = FISTA_Forest(b{t}, RR{t}, f(:,t), input);
    img(:,t) = out.y; x1(:,t)=Phi*img(:, t); 
    SNR3(:,t)=out.snr'; Time3(:,t)=out.xtime;
%     fprintf(1,'Iter_time = %.2fsec, Iteration Num = %d, snr = %f, samp.ratio = %f \n',out.xtime(end), input.no, out.snr(end), k/(m*n));
end
im3=reshape(img, [m, n, T]);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% Forest
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf('calling the function  FISTA_Forest.....\n');
input1=input; 
input1.alpha=alpha*sqrt(T);input1.beta=beta*sqrt(T);
out = FISTA_Forest(b, RR, f, input1);
out4=out;img = out.y; 
for t=1:T, 
    x4(:,t)=Phi*img(:,t); 
end
im4=reshape(img, [m, n, T]);
SNR4=out.snr'; Time4=out.xtime;
fprintf(1,'Iter_time = %.2fsec, Iteration Num = %d, snr = %f, samp.ratio = %f \n',out.xtime(end), input.no, out.snr(end), k/(m*n));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
SNR1=MT_SNR(SNR1, f); %SNR2=MT_SNR(SNR2, f);
SNR3=MT_SNR(SNR3, f);

Time1=sum(Time1, 2); %Time2=sum(Time2, 2);
Time3=sum(Time3, 2);


SNR(:, 1, iter)=SNR1; SNR(:,2, iter)=SNR2;
SNR(:, 3, iter)=SNR3; SNR(:,4, iter)=SNR4;

Time(:,1, iter)=Time1; Time(:,2, iter)=Time2;
Time(:,3, iter)=Time3; Time(:,4, iter)=Time4;

end;
SNR_M=mean(SNR, 3); Time_M=mean(Time, 3);
SNR_S=std(SNR,0, 3); Time_S=std(Time,0, 3);

ls=3; ms=1; ts=16;

figure; hold on;
plot(Time_M(:,1), SNR_M(:,1), 'b-.', 'linewidth',ls, 'markersize', ms);
plot(Time_M(:,2), SNR_M(:,2), 'g--', 'linewidth',ls, 'markersize', ms);
plot(Time_M(:,3), SNR_M(:,3), 'k:', 'linewidth',ls, 'markersize', ms);
plot(Time_M(:,4), SNR_M(:,4), 'r-', 'linewidth',ls, 'markersize', ms);
xlabel('CPU Time (s)');ylabel('SNR');box on;
legend( 'FISTA ','FISTA\_Joint','FISTA\_Tree','FISTA\_Forest', 'Location','SouthEast');
textobj = findobj('type', 'text'); set(textobj, 'fontsize', ts);
h_xlabel = get(gca,'XLabel'); set(h_xlabel,'FontSize',ts); 
h_xlabel = get(gca,'YLabel');set(h_xlabel,'FontSize',ts); 

 textobj = findobj('type', 'text');
set(textobj, 'fontsize', 14);
h_xlabel = get(gca,'XLabel');
set(h_xlabel,'FontSize',14); 
h_xlabel = get(gca,'YLabel');
set(h_xlabel,'FontSize',14); 

axis([0,26,22,36]);

figure;
subplot(2,2,1);imshow(abs(cat(2,im1(:,:,1),im1(:,:,2),im1(:,:,3))),[]); title('Standard');
subplot(2,2,2);imshow(abs(cat(2,im2(:,:,1),im2(:,:,2),im2(:,:,3))),[]); title('Joint');
subplot(2,2,3);imshow(abs(cat(2,im3(:,:,1),im3(:,:,2),im3(:,:,3))),[]); title('Tree');
subplot(2,2,4);imshow(abs(cat(2,im4(:,:,1),im4(:,:,2),im4(:,:,3))),[]); title('Forest');